"use strict";

function a2dom(arr) {
  let tag   = arr[0];  // 配列の最初の値がタグ名
  let attrs = arr[1];  // 2番目の値が属性（名前と値）
  let pos   = 2;       // 3番目以降は子テキスト
  /// CSSのような「tag#id.class1.class2」という記法をサポート
  let id = null, cls = null;
  let m = tag.match(/^([-\w]+)(#[-\w]+)?(\..+)?/);
  if (m) { tag = m[1]; id = m[2]; cls = m[3]; }
  /// DOM要素を作成
  let elem = document.createElement(tag);
  if (id)  { elem.id = id.substr(1); }    // '#foo' -> 'foo'
  if (cls) { elem.className = cls.substr(1).replace('.', ' '); }
                                          // '.a.b.c' -> 'a b c'
  /// 属性を追加（nullとundefinedは無視）
  let a = attrs;
  if (typeof a === 'object' && a != null && !$.isArray(a)) {
    for (let k in attrs) {
      let v = attrs[k];
      if (v === true || v === false) { // 真偽値なら（例：checked）
        elem[k] = v;                   //   プロパティとして設定
      } else if (v != null) {          // nullやundefinedでないなら
        elem.setAttribute(k, v);       //   属性値として設定
      }
    }
  } else {
    pos = 1; // 2番目が属性ではなかった ＝ 2番目以降が子テキスト
  }
  /// 配列の残りの値を、子ノードとして追加
  for (let i = pos, n = arr.length; i < n; i++) {
    let x = arr[i];
    if ($.isArray(x)) {            // 値が配列なら
      let child = a2dom(x);        //   a2dom()を再帰的に呼び出して
      elem.appendChild(child);     //   子要素として追加
    } else if ($.isFunction(x)){   // 値が関数なら
      let children = [];           //   新しい配列を作成し
      x(children);                 //   引数にして関数を呼び出す
      let rest = arr.slice(i+1);   //   残りの値を取り出す
      arr = children.concat(rest); // 新しいループ対象に切り替える
      i = -1; n = arr.length;      //   ループ変数をリセット
    } else if (x != null) {        // 値がnullやundefinedでないなら
      let text = document.createTextNode(x);
      elem.appendChild(text);      //   テキストを追加
    }
  }
  /// '<a>' を '<a href="javascript:undefined">' に変換
  if (elem.tagName === 'A' && !elem.href) {
    elem.setAttribute('href', 'javascript:undefined');
  }
  /// 作成したDOM要素を返す
  return elem;
}

/// for Node.js
if (typeof module !== 'undefined') { module.exports = a2dom; }
