"use strict";

let ConcentrationGame = (function() {

  /// ユーティリティ
  let U = {
    init: function(reset) {
    }
    /// 配列をn個ずつに切り出す。例：
    ///   chunk([1,2,3,4,5], 2); //=> [[1,2],[3,4],[5]]
  , chunk: function chunk(arr, n) {
      let rows = [];
      for (let i = 0, len = arr.length; i < len; i += n) {
        let row = arr.slice(i, i + n);
        rows.push(row);
      }
      return rows;
    }
  };

  /// モデル（コアとなるロジックやデータ）
  let M = {
    init: function(reset) {
      M.boardSize = 4;            // ボードの一辺の大きさ
      M.board = M.createBoard();  // ボード（4×4のマス目）を生成
    }
    /// ボードを生成する
  , createBoard: function() {
      // 1〜8までの数字をもつ配列を生成（4×4の場合）
      let n = M.boardSize;
      let nums = _.range(1, n * n / 2 + 1);
      // 自分自身を連結して、長さ16の配列を生成
      // （神経衰弱なので、同じ数字が2組ずつ必要）
      nums = nums.concat(nums);
      // ランダムに並び替え
      nums = _.shuffle(nums);
      // 数値を4個ずつ4回取り出して、4×4のマス目を生成
      let board = U.chunk(nums, n);
      return board;
    }
  };

  /// コントローラ（イベントハンドラ）
  let C = {
    init: function(reset) {
      if (! reset) {           // リセット時の重複登録を避ける
        C.registerHandlers();  // イベントハンドラを登録
      }
      V.renderBoard(M.board);  // ボードを表示
    }
    /// イベントハンドラを登録
  , registerHandlers: function() {
      $('#reset').on('click', C.doReset);  // リセットボタン
    }
    /// 全体をリセット
  , doReset: function() {
      if (confirm("Reset?")) {    // ダイアログでユーザに確認し、
        init(true);               // OKならUとMとVとCをリセット
      }
    }
  };

  /// ビュー（表示に関する機能）
  let V = {
    init: function(reset) {
    }
    /// ボードを表示
  , renderBoard: function(board) {
      let context = {
        board: board
      };
      $('#tmpl-board').renderWith(context, '#board');
    }
  };

  /// 初期設定
  function init(reset) {
    U.init(reset);
    M.init(reset);
    V.init(reset);
    C.init(reset);
  }

  return {init: init, U: U, M: M, C: C, V: V};

})();

$(function() {
  ConcentrationGame.init();
});
