"use strict";

/// テンプレートリテラル用タグ関数
function curet(strs, ...values) {
  let i = 0, buf = '', escape = curet.escape;
  for (let n = values.length; i < n; i++) {
    let s = strs[i], v = values[i];
    buf += s; buf += escape(v);
  }
  buf += strs[i];
  return buf;
}

/// HTMLエスケープ
curet.escape = function escape(x) {
  if (x == null) return "";
  return (""+x).replace(/[&<>"']/g, curet._quote);
};
curet._quote = function(m) { return curet._entities[m[0]]; };
curet._entities = {'&': '&amp;', '<': '&lt;', '>': '&gt;',
                   '"': '&quot;', "'": '&#39;'};

/// テンプレートリテラル（curet`...`）を作成
curet.literalize = function literalize(text) {
  let s = text.replace(/\\/g, "\\\\").replace(/`/g, "\\`");
  return "curet`" + s + "`";
};

/// 文のパターンを表す正規表現
curet.STATEMENT_REGEXP = /(^[ \t]*)?<!--%([\s\S]*?)%-->(\n?)/mg;

/// テンプレート中の「<!--% 文 %-->」をパースする
curet.parse = function parse(template) {
  let rexp = curet.STATEMENT_REGEXP;
  let code = "", pos = 0, m;
  while ((m = rexp.exec(template)) !== null) {
    let text = template.substring(pos, m.index);
    let space = m[1], stmt = m[2], newline = m[3];
    pos = m.index + m[0].length;
    if (text) { code += "_buf += " + curet.literalize(text) + ";"; }
    if (stmt) { code += (space||"") + stmt + (newline||""); }
  }
  let rest = pos === 0 ? template
                       : template.substring(pos, template.length);
  if (rest) { code += "_buf += " + curet.literalize(rest) + ";"; }
  return code;
};

/// テンプレート文字列をコンパイルし、関数を返す
curet.compile = function compile(template) {
  let code = "(function(){ var _buf = '';" +
             curet.parse(template) +
             "; return _buf;})";
  if (curet.debug) { console.log('[curet] debug: compiled=', code);}
  let func = eval(code);
  return func;
};

/// デバッグモード用フラグ
/// （オンならコンパイルされたコードをconsoleに出力）
curet.debug = false;

/// for Node.js
if (typeof module !== 'undefined') { module.exports = curet; }
