// -*- coding: utf-8 -*-
"use strict";

(function(exports) {

  class MartinRouter {

    constructor() {
      this._entries = [];
    }

    mount(pathpat, func) {
      let a = this.compile(pathpat);
      let x = {path: pathpat, func: func,
               rexp: a[0], params: a[1]};
      this._entries.push(x);
      return this;
    }

    route(path) {
      if (! path) return null;
      for (let x of this._entries) {
        let m = x.rexp.exec(path);
        if (m) {
          let args = m.slice(1, m.length);
          return [x.func, args];
        }
      }
      return null;
    }

    compile(pathpat) {
      let pathrexp = /(.*?)(:\w+)/g;
      let buf = '^', params = [], pos = 0, m;
      while ((m = pathrexp.exec(pathpat)) != null) {
        let text = m[1], param = m[2];
        pos = m.index + m[0].length;
        buf += this._escapeRexp(text);
        buf += '([^/]+)';
        params.push(param.substr(1));  // remove ':'
      }
      let rest = pos === 0 ? pathpat : pathpat.substr(pos);
      buf += this._escapeRexp(rest);
      buf += '$';
      return [new RegExp(buf), params];
    }

    _escapeRexp(s) {
      return s.replace(/[.*+?|^$(){}\[\]\\]/g, '\\$&');
    }

  }

  if (typeof module !== 'undefined') {
    module.exports = MartinRouter;
  } else {
    window.MartinRouter = MartinRouter;
  }

})();

/*

let router = new exports.MartinRouter();

if (typeof window !== 'undefined') {
  $(window).on('hashchange', function() {
    let path = window.location.hash.substr(1);  // remove '#'
    let func = router.route(path);
    if (func == null) {
      func = router.fail;
    }
    func();
  });
}


if (true) {
  let fn1 = function() { console.log('** fn1'); };
  let fn2 = function() { console.log('** fn2'); };
  let fn3 = function() { console.log('** fn3'); };
  let fn4 = function(id, id2) { console.log('** fn4: id2=', id2); return id2; };

  router.mount('/all'       , fn1);
  router.mount('/active'    , fn2);
  router.mount('/completed' , fn3);
  router.mount('/todo/:id/:id2.html'  , fn4);
  console.log(router.route('/todo/123/xy.html'));
  let ret = router.route('/todo/123/xy.html');
  let fn = ret[0], args = ret[1];
  console.log("*** debug: fn=", fn);
  console.log("*** debug: args=", args);
  let ret2 = fn.apply(null, args);
  console.log("*** debug: ret2=", ret2);
  //
  let path = '/todo/123';
  let pathpat = '/todo/:id/comments/:comment_id';
  let rexp = /(.*?)(:\w+)/g, m;
  let pos = 0;
  let buf = '^';
  let escaperexp = function(s) {
    return s.replace(/[^$*+?.()|{}\[\]\\]/g, '\\$&');
  };
  while ((m = rexp.exec(pathpat)) != null) {
    let text = m[1], param = m[2];
    if (text) { buf += escaperexp(text); }
    buf += '([^/]+)';
    pos = m.index + m[0].length;
    console.log([text, param, pos]);
  }
  let rest = pathpat.substr(pos);
  if (rest) { buf += escaperexp(rest); }
  buf += '$';
  console.log("*** debug: buf=", buf);
  let re = new RegExp(buf);
  console.log("*** debug: re=", re);
}
*/
